@@ -1,28 +1,19 @@
 """
 Exceptions for pyproj
 """
+from pyproj._datadir import _clear_proj_error, _get_proj_error
 
 
 class ProjError(RuntimeError):
     """Raised when a Proj error occurs."""
 
-    internal_proj_error = None
-
     def __init__(self, error_message: str) -> None:
-        if self.internal_proj_error is not None:
-            error_message = (
-                f"{error_message}: (Internal Proj Error: {self.internal_proj_error})"
-            )
-            ProjError.clear()
+        proj_error = _get_proj_error()
+        if proj_error is not None:
+            error_message = f"{error_message}: (Internal Proj Error: {proj_error})"
+            _clear_proj_error()
         super().__init__(error_message)
 
-    @staticmethod
-    def clear() -> None:
-        """
-        This will clear the internal PROJ erro message.
-        """
-        ProjError.internal_proj_error = None
-
 
 class CRSError(ProjError):
     """Raised when a CRS error occurs."""
